#!/home/fermiuser/work/scripts/py27

import pyfits
import re
import os
import subprocess
import string
from numpy import arccos,cos,argsort,zeros,arange,sin

# Author: Nikolai Shaposhnikov

# Notes:
# 2013-03-25:
#  psfcorrect adjusts countrates in channels according
# to the psf spillover calculated by integrating the
# psf from theta =0 to theta corresponding to the data
# extraction radius (parameter r). The method is not
# entirely appropriate and now obsolete.  Now I use
# arfcreate to produce a separate arf file which properly 
# accounts for psf spillover .

# Usage:
# arfcreate(arffile,rad,rspfile,psffile)
#     arffile - output file containing arf matrix
#     rad - pha file extraction radius, i.e. the radius for
#           which the spillover is calculated
#     rspfile - LAT response file, produced by gtrspgen
#     psffile - LAT psf file file, produced by gtpsf


def psfcorrect(specf,outfile,r,psff):

    pi = 2.0*arccos(0.0)
    os.system("rm -f "+outfile)
    os.system("fcalc "+specf+" "+outfile+" PSF COUNTS*0.0")
    os.system("fcalc "+outfile+" !"+outfile+" COUNTS_RAW COUNTS*1.0")
    
    psffil = pyfits.open(psff)
    sfil = pyfits.open(outfile,mode='update')
    psfspec = sfil[1].data.field('PSF')
    psf = psffil[1].data.field('Psf')
    psf_ener = psffil[1].data.field('Energy')
    theta = psffil[2].data.field('Theta')
    dtheta = theta[1]-theta[0]
    spectrum = sfil[1].data.field('COUNTS')
    stat_err = sfil[1].data.field('STAT_ERR')
    numchan = len(spectrum)
    sp = sfil[1].data.field('COUNTS_RAW')
    if len(psf_ener) != numchan+1 : 
        print "Warning from psfcorrect!"
        print " Array length mismatch between psf ("+str(len(psf_ener))+" energies) and number of  PHA channels ("+str(numchan)+" channels)"
    
    print dtheta,sin(pi),sin(2.0*pi)
    for i in range(numchan):
        cont = 0.0
        for k in range(len(theta)-1): 
            if theta[k]<roi:
                cont += (psf[i,k]+psf[i,k+1]+psf[i+1,k]+psf[i+1,k+1])*0.5*pi*(cos(pi*theta[k]/180.0)-cos(pi*theta[k+1]/180.0))
        psfspec[i] = cont
        spectrum[i] = sp[i]/cont
        stat_err[i] = stat_err[i]/sqrt(cont)
#        print i,psf_ener[i],sp[i],spectrum[i],cont
    
#    sfil.flush()
    sfil.close()
    psffil.close()

#def sdist(r,d,raobj,decobj):
#    pi2 = arccos(0.0)/90.0
#    return arccos(cos(pi2*(r-raobj))*cos(pi2*(d-decobj)))/pi2 


def arfcreate(outfile,roi,rspf,psff):

    import warnings
    
    
    warnings.filterwarnings('ignore')
    res_str =  "This is ARFCREATE: ancillary response creator for Fermi/LAT spectrum.\n"
    res_str += "   outfile       = "+outfile+"\n"
    res_str += "   roi           = "+str(roi)+"\n"
    res_str += "   response file = "+rspf+"\n"
    res_str += "   PSF file      = "+psff+"\n"

    r = roi
    pi = 2.0*arccos(0.0)
    try:
        try:
            os.system("rm -f "+outfile)
        except:
            pass
    
        psffil = pyfits.open(psff)
        rspfil = pyfits.open(rspf)
        energ_lo = rspfil[1].data.field('ENERG_LO')
        energ_hi = rspfil[1].data.field('ENERG_HI')
        psf = psffil[1].data.field('Psf')
        psf_ener = psffil[1].data.field('Energy')
        theta = psffil[2].data.field('Theta')
        dtheta = theta[1]-theta[0]
#    spectrum = sfil[1].data.field('COUNTS')
#    stat_err = sfil[1].data.field('STAT_ERR')
#    numchan = len(spectrum)
#    sp = sfil[1].data.field('COUNTS_RAW')
#    if len(psf_ener) != numchan+1 : 
#        print "Warning from psfcorrect!"
#        print " Array length mismatch between psf ("+str(len(psf_ener))+" energies) and number of  PHA channels ("+str(numchan)+" channels)"
    
#        print dtheta,sin(pi),sin(90.0)
        arf = zeros(len(energ_lo),dtype=float)
        e_lo = zeros(len(energ_lo),dtype=float)
        e_hi = zeros(len(energ_lo),dtype=float)
        for i in range(len(energ_lo)):
            cont = 0.0
            for k in range(len(theta)-2): 
                if theta[k]<roi:
                #                cont += psf[i,k]*2.0*pi*(cos(pi*theta[k]/180.0)-cos(pi*theta[k+1]/180.0))
                    cont += (psf[i,k]+psf[i,k+1]+psf[i+1,k]+psf[i+1,k+1])*0.5*pi*(cos(pi*theta[k]/180.0)-cos(pi*theta[k+1]/180.0))
#                cont += (psf[i,k]+psf[i,k+1]+psf[i+1,k]+psf[i+1,k+1])*0.5*pi*(sin(pi*(theta[k]+theta[k+1])/360.0)*(theta[k+1]-theta[k])/360.0)
            arf[i] = cont
            e_lo[i] = energ_lo[i]
            e_hi[i] = energ_hi[i]
#        print i,psf_ener[i],sp[i],spectrum[i],cont
#    sfil.flush()

        arf_col = pyfits.Column(name = 'SPECRESP',format='E',array = arf)
        elo_col = pyfits.Column(name = 'ENERG_LO',format='E',array = e_lo)
        ehi_col = pyfits.Column(name = 'ENERG_HI',format='E',array = e_hi)
        cols = pyfits.ColDefs([elo_col,ehi_col,arf_col])
        prhdu = pyfits.PrimaryHDU()
        tbhdu = pyfits.new_table(cols)
#    tbhdu.header.update('TUNIT1','keV     ')
#    tbhdu.header.update('TUNIT2','keV     ')
#    tbhdu.header.update('TUNIT3','cm**2   ')

        prhdu.header.update('NAXIS',0)
        tbhdu.header.update('TUNIT1','keV     ',after='TFORM1')
        tbhdu.header.update('TUNIT2','keV     ',after='TFORM2')
        tbhdu.header.update('TUNIT3','cm**2   ',after='TFORM3')

        tbhdu.header.update('EXTNAME','SPECRESP')
        tbhdu.header.update('HDUCLASS','OGIP    ')
        tbhdu.header.update('HDUCLAS1','RESPONSE')
        tbhdu.header.update('HDUCLAS2','SPECRESP')
        tbhdu.header.update('HDUVERS','1.1.0   ')
        tbhdu.header.update('HDUVERS1','1.0.0   ')
        tbhdu.header.update('HDUVERS2','1.1.0   ')
        tbhdu.header.update('TELESCOP','FERMI   ')
        tbhdu.header.update('INSTRUME','LAT     ')
        tbhdu.header.update('DETNAM','LAT     ')
        tbhdu.header.update('FILTER','NONE    ')
#        tbhdu.header.update('RESPFILE',rspf)
        tbhdu.header.update('PHAFILE','')
        hdulist = pyfits.HDUList([prhdu,tbhdu])
        hdulist.writeto(outfile)

        rspfil.close()
        psffil.close()

    except:
        res_str += "***!!!!****Error occured during ARF calculation.\n"
    else:
        res_str += "Done.\n"        
    return res_str





